#!/bin/bash
# Script to install a prebuilt libraries into install folder
#
# The default platform is LinuxFB only
#
# To use another, please define :
# USE_WL="yes" to enable Wayland support;  USE_WL="no" if support is not required
# USE_X11="yes" to enable X11 support;  USE_X11="no" if support is not required
#
# For 2D-only GPU, define IMXGPU="2d"

if [ -r "$PLATFORM_LOC/scripts/utils.sh" ]; then
	source "$PLATFORM_LOC/scripts/utils.sh"
else
	source "$script_lib_dir/utils.sh"
fi

rm -rf _install
mkdir -p _install

S=$PWD
D=$install_loc

libdir=/usr/lib
includedir=/usr/include
bindir=/usr/bin
sysconfdir=/etc

do_buildstep_prepend

log_exec install -d ${D}${libdir}
log_exec install -d ${D}${includedir}
log_exec install -d ${D}${bindir}

log_exec cp -P ${S}/gpu-core/usr/lib/*.so* ${D}${libdir}
log_exec cp -r ${S}/gpu-core/usr/include/* ${D}${includedir}
log_exec cp -r ${S}/gpu-demos/opt ${D}
log_exec cp -r ${S}/gpu-tools/gmem-info/usr/bin/* ${D}${bindir}

# Use vulkan header from vulkan recipe to support vkmark
rm -rf ${D}${includedir}/vulkan/

log_exec install -d ${D}${libdir}/pkgconfig
if ${HAS_GPU3D}; then
	if [ -f "${S}/gpu-core/usr/lib/pkgconfig/gbm.pc" ]; then
    	log_exec install -m 0644 ${S}/gpu-core/usr/lib/pkgconfig/gbm.pc ${D}${libdir}/pkgconfig/gbm.pc
    fi
fi

# The preference order, based in DISTRO_FEATURES, is Wayland (with or without X11), X11 and fb
if [ "${USE_WL}" = "yes" ]; then

    backend=wl

    log_exec install -m 0644 ${S}/gpu-core/usr/lib/pkgconfig/egl_wayland.pc ${D}${libdir}/pkgconfig/egl.pc
    log_exec install -m 0644 ${S}/gpu-core/usr/lib/pkgconfig/glesv1_cm.pc ${D}${libdir}/pkgconfig/glesv1_cm.pc
    log_exec install -m 0644 ${S}/gpu-core/usr/lib/pkgconfig/glesv2.pc ${D}${libdir}/pkgconfig/glesv2.pc
    log_exec install -m 0644 ${S}/gpu-core/usr/lib/pkgconfig/vg.pc ${D}${libdir}/pkgconfig/vg.pc

	if [ "${USE_X11}" = "yes" ]; then
		log_exec cp -r ${S}/gpu-core/usr/lib/dri ${D}${libdir}
	fi

elif [ "${USE_X11}" = "yes" ]; then

    log_exec cp -r ${S}/gpu-core/usr/lib/dri ${D}${libdir}

    backend=x11

    log_exec install -m 0644 ${S}/gpu-core/usr/lib/pkgconfig/gl_x11.pc ${D}${libdir}/pkgconfig/gl.pc
    log_exec install -m 0644 ${S}/gpu-core/usr/lib/pkgconfig/egl_x11.pc ${D}${libdir}/pkgconfig/egl.pc
    log_exec install -m 0644 ${S}/gpu-core/usr/lib/pkgconfig/glesv1_cm_x11.pc ${D}${libdir}/pkgconfig/glesv1_cm.pc
    log_exec install -m 0644 ${S}/gpu-core/usr/lib/pkgconfig/glesv2_x11.pc ${D}${libdir}/pkgconfig/glesv2.pc
    log_exec install -m 0644 ${S}/gpu-core/usr/lib/pkgconfig/vg_x11.pc ${D}${libdir}/pkgconfig/vg.pc

else
    log_exec install -m 0644 ${S}/gpu-core/usr/lib/pkgconfig/glesv1_cm.pc ${D}${libdir}/pkgconfig/glesv1_cm.pc
    log_exec install -m 0644 ${S}/gpu-core/usr/lib/pkgconfig/glesv2.pc ${D}${libdir}/pkgconfig/glesv2.pc
    log_exec install -m 0644 ${S}/gpu-core/usr/lib/pkgconfig/vg.pc ${D}${libdir}/pkgconfig/vg.pc

    # Regular framebuffer
    log_exec install -m 0644 ${S}/gpu-core/usr/lib/pkgconfig/egl_linuxfb.pc ${D}${libdir}/pkgconfig/egl.pc
    
    backend=fb
fi

# Install Vendor ICDs for OpenCL's installable client driver loader (ICDs Loader)
log_exec install -d ${D}${sysconfdir}/OpenCL/vendors/
log_exec install -m 0644 ${S}/gpu-core/etc/Vivante.icd ${D}${sysconfdir}/OpenCL/vendors/Vivante.icd

# We'll only have one backend here so we rename it to generic name
# and avoid rework in other packages, when possible
log_exec mv ${D}${libdir}/libGL.so.1.2 ${D}${libdir}/libGL.so.1.2.0
log_exec ln -sf libGL.so.1.2.0 ${D}${libdir}/libGL.so.1.2
log_exec ln -sf libGL.so.1.2.0 ${D}${libdir}/libGL.so.1
log_exec ln -sf libGL.so.1.2.0 ${D}${libdir}/libGL.so
log_exec mv ${D}${libdir}/libEGL-${backend}.so ${D}${libdir}/libEGL.so.1.0
log_exec ln -sf libEGL.so.1.0 ${D}${libdir}/libEGL.so.1
log_exec ln -sf libEGL.so.1.0 ${D}${libdir}/libEGL.so
log_exec mv ${D}${libdir}/libGAL-${backend}.so ${D}${libdir}/libGAL.so

if [ "${USE_WL}" = "yes" ]; then
    # Special case for libVDK on Wayland backend, deliver fb library as well.
    # Need to rename the libraries to avoid the code below that will remove
    # *-fb.so and *-wl.so
    log_exec mv ${D}${libdir}/libVDK-fb.so ${D}${libdir}/libVDK-framebuffer.so
    log_exec mv ${D}${libdir}/libVDK-wl.so ${D}${libdir}/libVDK-wayland.so
    log_exec ln -sf libVDK-wayland.so ${D}${libdir}/libVDK.so
else
    log_exec mv ${D}${libdir}/libVDK-${backend}.so ${D}${libdir}/libVDK.so
fi

# update libglesv2 as backend dependent
log_exec rm -rf ${D}${libdir}/libGLESv2*
log_exec cp ${S}/gpu-core/usr/lib/libGLESv2-${backend}.so ${D}${libdir}/libGLESv2.so.2.0.0
log_exec ln -sf libGLESv2.so.2.0.0 ${D}${libdir}/libGLESv2.so.2
log_exec ln -sf libGLESv2.so.2.0.0 ${D}${libdir}/libGLESv2.so

if [ "${IS_MX8}" = "1" ]; then
    # Install the vulkan driver in a sub-folder. When installed in the same
    # folder as the vulkan loader layer library, an incorrect linkage is
    # created from libvulkan.so.1 to our library instead of the loader
    # layer library.
    log_exec install -d ${D}${libdir}/vulkan
    log_exec mv ${D}${libdir}/libvulkan-${backend}.so ${D}${libdir}/vulkan/libvulkan_VSI.so
fi

# skip packaging wayland libraries if no support is requested
if [ "${USE_WL}" = "no" ]; then
    log_exec "find ${D}${libdir} -name "libgc_wayland_protocol.*" -exec rm '{}' ';'"
    log_exec "find ${D}${libdir} -name "libwayland-viv.*" -exec rm '{}' ';'"
fi

for i in wl x11 fb dri; do
    log_exec "find ${D}${libdir} -name "*-$i.so" -exec rm '{}' ';'"
    log_exec "find ${D}${libdir} -name "*.$i.so" -exec rm '{}' ';'"
done


# FIXME: MX6SL does not have 3D support; hack it for now
if [ "${IS_MX6SL}" = "1" ]; then
    log_exec rm -rf ${D}${libdir}/libCLC* ${D}${includedir}/CL \
           \
           ${D}${libdir}/libGL* ${D}${includedir}/GL* ${D}${libdir}/pkgconfig/gl.pc \ 
           \
           ${D}${libdir}/libGLES* ${D}${libdir}/pkgconfig/gles*.pc \
           \
           ${D}${libdir}/libOpenCL* ${D}${includedir}/CL \
           \
           ${D}${libdir}/libOpenVG.3d.so \
           \
           ${D}${libdir}/libVivanteOpenCL.so \
           \
           ${D}/opt/viv_samples/vdk \
           ${D}/opt/viv_samples/es20 ${D}/opt/viv_samples/cl11

    log_exec ln -sf libOpenVG.2d.so ${D}${libdir}/libOpenVG.so
fi

log_exec "find ${D}${libdir} -type f -exec chmod 644 {} \;"
log_exec "find ${D}${includedir} -type f -exec chmod 644 {} \;"

log_exec chown -R root:root ${D}/*

log "Libraries succesfully deployed"

do_buildstep_append
